#define _UNICODE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>

/*
pg2 -- 
    Display Data from Standard Input in Notepad or Less
    Copyright (C) 2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com
	Compile with Visual Studio
   
*/

Usage() {
	FILE *hlp;
	hlp = stderr;
		fprintf(hlp,"PG2 Version 1.03, %s\n",__DATE__);
		fprintf(hlp,"Copyright 2003, 2009 Norm London, London Computing.  norm_london@londoncomputing.com\n\n");
        fprintf(hlp,"This program is free software; you can redistribute it and/or modify\n");
		fprintf(hlp,"it under the terms of the GNU General Public License as published by\n");
		fprintf(hlp,"the Free Software Foundation; either version 2 of the License, or\n");
		fprintf(hlp,"(at your option) any later version.\n\n");
		fprintf(hlp,"This program is distributed in the hope that it will be useful,\n");
		fprintf(hlp,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
		fprintf(hlp,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
		fprintf(hlp,"GNU General Public License for more details.\n\n");
		fprintf(hlp,"You should have received a copy of the GNU General Public License\n");
		fprintf(hlp,"along with this program; if not, write to the Free Software\n");
		fprintf(hlp,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");
		fprintf(stderr,"Usage is: pg2 [-q] [filename]\n");
		fprintf(stderr,"Reads input file and displays in NOTEPAD with line numbers and a ruler line.\n");
		fprintf(stderr,"If filename omitted, reads from the console.\n");
		fprintf(stderr,"If the file is changed, then the numbers and rulers are stripped out.\n");
		fprintf(stderr,"-Q -- omit rulers and line numbers. (Default)\n");
		fprintf(stderr,"-R -- display rulers and line numbers. \n");
		return 4;
}

int wmain(int argc,wchar_t *argv[]) {
	// Page through std input
	// Add line numbers
	// invoke notepad
	
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
    wchar_t fname[_MAX_FNAME];
    wchar_t ext[_MAX_EXT];
	char quiet = 1;

	struct _stat before, after;
	wchar_t inputline[2048], filename[_MAX_DIR+_MAX_DIR+_MAX_FNAME+_MAX_EXT+32], *b;
 	wchar_t ruler[]=L"....+....1....+....2....+....3....+....4....+....5....+....6....+....7....+....8....+....9....+....A....+....B....+....C....+....D....+....E....+....F....+....G....+....H....+....I....+....J....+....K....+....L....+....M....+....N....+....O....+....P....+";
	wchar_t *tempaddr;
	size_t tempaddrlen;

	FILE *out,*in = NULL;
	int returncode, rc;
	long lineno=0;
	size_t inputlinesize, filenamesize;
	int arg = 1;

	inputlinesize = sizeof(inputline)/sizeof(wchar_t);

	filenamesize = sizeof(filename)/sizeof(wchar_t);
	if (argc > 1) {
		if (_wcsicmp(argv[1],L"-Q")==0) {
			arg = 2;
			quiet = 1;
		}
		if (_wcsicmp(argv[1],L"-R")==0) {
			arg = 2;
			quiet = 0;
		}
	}

	if (argc > arg) {
		// If ? in argument, print help
		if (wcschr(argv[arg],L'?')) {
			Usage();
			return 4;
		}
		// no ?
		_wsplitpath(argv[arg],drive,dir,fname,ext);
		rc = _wfopen_s(&in,argv[arg],L"r");
		if (rc) {
			fprintf(stderr,"Error %d, Unable to open input file: %ls\n",rc,in);
			exit(4);
		}
		rc = _wdupenv_s(&tempaddr,&tempaddrlen,L"TMP");
		if (rc) {
			fprintf(stderr,"Error %d, unable to locate environment variable: TMP\n",rc);
			exit(4);
		}
		wcscpy_s(filename,filenamesize,tempaddr);
		wcscat_s(filename,filenamesize,L"/PG2_");
		wcscat_s(filename,filenamesize,fname);
		wcscat_s(filename,filenamesize,ext);
		if (in == NULL) {
			fprintf(stderr,"Unable to open: %ls\n",argv[1]);
			exit(4);
		}
	}
	else {
		if (_isatty(_fileno(stdin))) {
			// File not redirected, just output usage
			Usage();
			return 4;
		}
		in = stdin;
		wcscpy_s(filename,filenamesize,_wtempnam(L"c:\\",L"pg2_"));
	}

	returncode = _fstat(_fileno(in),&before);
	if (returncode == -1) {
		fprintf(stderr,"Input not redirected or file is empty.\n");
		Usage();
		return 4;
	}

	rc = _wfopen_s(&out,filename,L"w");

	if (rc || out == NULL) {
		fprintf(stderr,"Error %d, Unable to allocate temporary file: %ls\n",rc,filename);
		return 4;
	}
	
	while (fgetws(inputline,sizeof(inputline),in) != NULL) {
		if (!quiet  && (lineno % 20) == 0)
			fprintf(out,"      : %ls\n",ruler);
		if (!quiet )
			fprintf(out,"%6d: ",++lineno);
		fputws(inputline,out);
	}

	fclose(out);
	if (in != stdin)
		fclose(in);

	inputlinesize = sizeof(inputline)/sizeof(wchar_t);
	wcscpy_s(inputline,inputlinesize,L"NOTEPAD ");
	wcscat_s(inputline,inputlinesize,filename);

	_wstat(filename,&before);
	returncode = _wsystem(inputline);

	if (returncode == -1)
		fprintf(stderr,"%ls -- failed.\n",inputline);
	_wstat(filename,&after);

	if (before.st_mtime < after.st_mtime) {
		// Strip out line numbers and rulers
		if (argc < 2) {
			// reading from standard input
			printf("Enter filename for modified output: ");
			fgetws(inputline,sizeof(inputline),stdin);
			inputline[wcslen(inputline)-1] = 0;
		}
		else 
			wcscpy_s(inputline,sizeof(inputline),argv[1]);
		rc = _wfopen_s(&out,inputline,L"w");

		if (out == NULL) {
			fprintf(stderr,"Unable to open output file: %ls\n",inputline);
			fprintf(stderr,"Writing to stdout.\n");
			out = stdout;
		}
		
		rc = _wfopen_s(&in,filename,L"r");
		if (in == NULL) {
			fprintf(stderr,"Unable to open: %ls\n",filename);
			return 4;
		}
		
		while(fgetws(inputline,sizeof(inputline),in)) {
			// remove line numbers and rulers
			b = wcschr(inputline,L':');
			if (b == NULL)
				b = inputline;
			else
				b += 2;
			if (wcsncmp(b,ruler,wcslen(ruler)) != 0)
				fputws(b,out);
		}

		fclose(in);
		fclose(out);
	}

	_wunlink(filename);
	return returncode;
}


	
	